import os
import shutil
import time
import h5py
import matplotlib.pyplot as plt
import single_pattern as single
import multi_pattern as multi
from math import log10, ceil
import periodical as period
import always_on as on
import on_off_pattern
import plot as plot
import noise_generators as gnoise
import timestamp as tmstp

import numpy as np
import pandas as pd

import glob, gc
import yaml
import shutil
import json
import os

from datetime import datetime
from periodical import fridge
from always_on import router

from single_pattern import *
from on_off_pattern import *
from multi_pattern import *

from HDF5_converter import convert_SynD

samples_per_day = 864000
log_file = 'synd_log.txt'

appID = {
    # SMART METER SIGNAL
    "aggregate": 1,

    # PERIODICAL APPLIANCES
    "fridge": 2,

    # MULTI PATTERN APPLIANCES
    "dishwasher": 3,
    "heater": 4,
    "washing machine": 5,
    "toaster": 6,
    "fan": 7,
    "microwave": 8,
    "iron": 9,
    "hot air gun": 10,

    # ALWAYS ON
    "router": 11,

    # ON-OFF APPLIANCES
    "coffee machine": 12,
    "TV": 13,

    "printer": 14,
    "laptop": 15,
    "lamp": 16,
    "gaming PC": 17,

    # SINGLE PATTERN
    "radio": 18,
    "monitor": 19,
    "minioven": 20,
    "hair dryer": 21,
    "watercooker": 22
}

appMapper = {
    # PERIODICAL APPLIANCES
    "fridge": (lambda x: fridge(x)),

    # MULTI PATTERN APPLIANCES
    "dishwasher": (lambda x: dishwasher(x)),
    "heater": (lambda x: heater(x)),
    "washing machine": (lambda x: washingmachine(x)),
    "toaster": (lambda x: toaster(x)),
    "fan": (lambda x: fan(x)),
    "microwave": (lambda x: microwave(x)),
    "iron": (lambda x: iron(x)),
    "hot air gun": (lambda x: hot_air_gun(x)),

    # ALWAYS ON
    "router": (lambda x: router(x)),

    # ON-OFF APPLIANCES
    "TV": (lambda x: TV(x)),
    "coffee machine": (lambda x: coffeemachine(x)),
    "printer": (lambda x: printer(x)),
    "laptop": (lambda x: laptop(x)),
    "lamp": (lambda x: lamp(x)),
    "gaming PC": (lambda x: gaming_PC(x)),

    # SINGLE PATTERN
    "radio": (lambda x: radio(x)),
    "monitor": (lambda x: monitor(x)),
    "minioven": (lambda x: minioven(x)),
    "hair dryer": (lambda x: hairdryer(x)),
    "watercooker": (lambda x: watercooker(x))
}


def load_settings(settings_file='settings.json'):
    with open(settings_file) as s_f:
        return json.load(s_f)


def clean_dir(dir):
    files = glob.glob(dir)
    for f in files:
        os.remove(f)


def remove_csv(dir):
    path = os.listdir(dir)
    for item in path:
        if item.endswith('.csv'):
            os.remove(os.path.join(dir, item))


def update_metadata(settings_dict):

    with open('metadata/dataset.yaml', 'r') as outfile:
        meta_dict = yaml.load(outfile)

    meta_dict['dataset_ID'] = settings_dict['ID']
    meta_dict['duration'] = settings_dict['nr_days']
    meta_dict['sampling_interval'] = settings_dict['sampling_interval']
    meta_dict['date_created'] = datetime.now().date().strftime('%Y-%m-%d')

    with open('metadata/dataset.yaml', 'w') as outfile:
        yaml.dump(meta_dict, outfile, default_flow_style=False)


    with open('metadata/meter_devices.yaml', 'r') as outfile:
        meta_dict = yaml.load(outfile)

    meta_dict['HMC8015']['sample_period'] = settings_dict['sampling_interval']
    meta_dict['HMC8015']['max_sample_period'] = settings_dict['sampling_interval']

    with open('metadata/meter_devices.yaml', 'w') as outfile:
        yaml.dump(meta_dict, outfile, default_flow_style=False)

    return 0


def load_clean_resample_series(app_name, settings_dict):
    x = np.array([])

    for i in range(settings_dict['nr_days']):
        x = np.concatenate([x, appMapper[app_name](samples_per_day)])

    df = pd.Series(x)
    tz_naive = pd.date_range(datetime.today().strftime('%Y-%m-%d'), periods=settings_dict['nr_days'] * samples_per_day,
                             freq='0.1S')

    tz_aware = tz_naive.tz_localize(tz=None)
    df.index = tz_aware

    df = df.replace([np.inf, -np.inf], np.nan)
    df = df.fillna(method='ffill')

    return df.resample('{}S'.format(settings_dict['sampling_interval'])).first()#median()


def _generate(settings_dict, basedir='target'):

    for app_name in settings_dict['appliances']:

        # Convert to Pandas Series and remove INFs and NaNs
        submeter_df = load_clean_resample_series(app_name, settings_dict)

        # Store to CSVs
        submeter_df.to_csv(
            'target/{}/{}.csv'.format(settings_dict['ID'], appID[app_name]), sep='\t', encoding='utf-8')

        try:

            aggregate_df = pd.Series.from_csv(
                'target/{}/{}.csv'.format(settings_dict['ID'], appID["aggregate"]), sep='\t', encoding='utf-8')

            aggregate_df += submeter_df

            aggregate_df.to_csv(
                'target/{}/{}.csv'.format(settings_dict['ID'], appID["aggregate"]), sep='\t', encoding='utf-8')

        except FileNotFoundError:
            submeter_df.to_csv(
                'target/{}/{}.csv'.format(settings_dict['ID'], appID["aggregate"]), sep='\t', encoding='utf-8')

        gc.collect()



if __name__ == '__main__':

    # LOAD SETTINGS
    settings = load_settings('settings.json')

    np.random.seed(settings['SEED'])

    try:
        os.mkdir('target/{}'.format(settings['ID']))

    except FileExistsError:
        pass

    # GENERATE DATA
    _generate(settings)

    if settings['output_format'] == 'CSV':

        shutil.copy('metadata.zip', 'target/{}'.format(settings['ID']))
        shutil.copy('appliance_labels.yml', 'target/{}'.format(settings['ID']))

        shutil.make_archive(
            'datasets/SynD_dataset_{}'.format(settings['ID']),
            'zip', 'target/{}'.format(settings['ID'])
        )

    elif settings['output_format'] == 'HDF5':

        # Convert to HDF5
        update_metadata(settings)

        convert_SynD('target/{}/'.format(settings['ID']), 'target/{}/SynD_{}.h5'.format(
            settings['ID'], settings['ID']), format='HDF')

        shutil.copy('metadata.zip', 'target/{}'.format(settings['ID']))
        shutil.copy('appliance_labels.yml', 'target/{}'.format(settings['ID']))

    else:
        pass

    exit()


'''
Copyright notice

CC0 - No Rights Reserved

Christoph Klemenjak (klemenjak@ieee.org), Christoph Kovatsch, Manuel Herold, and Wilfried Elmenreich

Find further material and the latest version of this tool at https://github.com/klemenjak/SynD/

The person who associated a work with this deed has dedicated the work to the public domain by waiving all of his or
her rights to the work worldwide under copyright law, including all related and neighboring rights, to the extent allowed by law.

You can copy, modify, distribute and perform the work, even for commercial purposes, all without asking permission.

Unless expressly stated otherwise, the person who associated a work with this deed makes no warranties about the work,
and disclaims liability for all uses of the work, to the fullest extent permitted by applicable law.
'''
