import json
import numpy as np
import pandas as pd
import os
import shutil
import time
import h5py
import matplotlib.pyplot as plt
import plot as plot
import interpolate as interpol
import time_handler as time_handle
import use_patterns as use_pat

day = 864000


def toaster(t):

    n_days = t//day
    data_final = np.zeros((t), dtype=None)
    for i in range(n_days):
        x = np.random.randint(1, 10)

        data, n_rows = use_pat.toaster_use(x)

        my = np.random.randint(290000, 340000)
        sigma = 10000
        my_we = np.random.randint(310000, 360000)
        sigma_we = 5000
        data_day = time_handle.time_handling(
            t, i, n_days, n_rows, my, sigma, data, n_rows, my_we, sigma_we, data)
        data_final += data_day

    data_final = np.transpose(data_final)

    return data_final


def washingmachine(t):

    n_days = t//day
    data_final = np.zeros((t), dtype=None)
    for i in range(n_days):
        x = np.random.randint(1, 10)

        data, n_rows = use_pat.washingmachine_use(x)

        my = np.random.randint(500000, 600000)
        sigma = 40000
        my_we = np.random.randint(360000, 620000)
        sigma_we = 20000
        data_day = time_handle.time_handling(
            t, i, n_days, n_rows, my, sigma, data, n_rows, my_we, sigma_we, data)
        data_final += data_day

    data_final = np.transpose(data_final)

    return data_final


def dishwasher(t):

    n_days = t//day
    data_final = np.zeros((t), dtype=None)
    for i in range(n_days):
        x = np.random.randint(1, 20)

        data, n_rows = use_pat.dishwasher_use(x)

        my = np.random.randint(450000, 600000)
        sigma = 50000
        my_we = np.random.randint(350000, 640000)
        sigma_we = 20000
        data_day = time_handle.time_handling(
            t, i, n_days, n_rows, my, sigma, data, n_rows, my_we, sigma_we, data)
        data_final += data_day

    data_final = np.transpose(data_final)

    return data_final


def fan(t):

    n_days = t//day
    data_final = np.zeros((t), dtype=None)
    for i in range(n_days):
        x = np.random.randint(1, 5)

        data, n_rows = use_pat.fan_use()


        t1 = np.random.randint(10000, 50000)
        interpolated = interpol.interpolate(t1, n_rows, data, x)
        my = 450000
        sigma = 80000

        t1_we = np.random.randint(10000, 50000)
        interpolated_we = interpol.interpolate(t1_we, n_rows, data, x)
        my1_we = 400000
        my2_we = 650000
        sigma_we = 20000


        s = np.random.randint(0, 2)
        if s == 0:
            data_day = time_handle.time_handling(
                t, i, n_days, t1, my, sigma, interpolated, t1_we, my1_we, sigma_we, interpolated_we)
            data_final += data_day
        else:
            data_day = time_handle.time_handling(
                t, i, n_days, t1, my, sigma, interpolated, t1_we, my2_we, sigma_we, interpolated_we)
            data_final += data_day

    data_final = np.transpose(data_final)

    return data_final


def heater(t):

    n_days = t//day
    data_final = np.zeros((t), dtype=None)
    for i in range(n_days):
        x = np.random.randint(1, 5)

        data, n_rows = use_pat.heater_use()

        t1 = np.random.randint(30000, 100000)
        interpolated = interpol.interpolate(t1, n_rows, data, x)
        my = np.random.randint(650000, 680000)
        sigma = 20000

        t1_we = np.random.randint(40000, 100000)
        interpolated_we = interpol.interpolate(t1_we, n_rows, data, x)
        my_we = np.random.randint(680000, 720000)
        sigma_we = 40000

        data_day = time_handle.time_handling(
            t, i, n_days, t1, my, sigma, interpolated, t1_we, my_we, sigma_we, interpolated_we)
        data_final += data_day

    data_final = np.transpose(data_final)

    return data_final


def iron(t):

    n_days = t//day
    data_final = np.zeros((t), dtype=None)
    for i in range(n_days):
        x = np.random.randint(1, 5)

        data, n_rows = use_pat.iron_use()

        t1 = np.random.randint(24000, 60000)
        interpolated = interpol.interpolate(t1, n_rows, data, x)
        my = np.random.randint(480000, 550000)
        sigma = 20000

        t1_we = np.random.randint(28000, 64000)
        interpolated_we = interpol.interpolate(t1_we, n_rows, data, x)
        my_we = np.random.randint(400000, 600000)
        sigma_we = 30000

        data_day = time_handle.time_handling(
            t, i, n_days, t1, my, sigma, interpolated, t1_we, my_we, sigma_we, interpolated_we)
        data_final += data_day

    data_final = np.transpose(data_final)

    return data_final


def hot_air_gun(t):

    n_days = t//day
    data_final = np.zeros((t), dtype=None)
    for i in range(n_days):
        x = np.random.randint(1, 4)

        data, n_rows = use_pat.hot_air_gun_use()

        t1 = np.random.randint(1800, 4200)
        interpolated = interpol.interpolate(t1, n_rows, data, x)
        my = np.random.randint(400000, 450000)
        sigma = 10000

        t1_we = np.random.randint(2400, 6000)
        interpolated_we = interpol.interpolate(t1_we, n_rows, data, x)
        my_we = np.random.randint(400000, 550000)
        sigma_we = 20000

        data_day = time_handle.time_handling(
            t, i, n_days, t1, my, sigma, interpolated, t1_we, my_we, sigma_we, interpolated_we)
        data_final += data_day

    data_final = np.transpose(data_final)

    return data_final


def microwave(t):

    n_days = t//day
    data_final = np.zeros((t), dtype=None)
    for i in range(n_days):

        data, n_rows = use_pat.microwave_use()

        t1 = np.random.randint(1200, 3000)
        interpolated = interpol.interpolate(t1, n_rows, data, 2)
        my = np.random.randint(590000, 640000)
        sigma = 10000

        t1_we = np.random.randint(1200, 3000)
        interpolated_we = interpol.interpolate(t1_we, n_rows, data, 2)
        my1_we = np.random.randint(590000, 640000)
        my2_we = np.random.randint(420000, 460000)
        sigma_we = 20000


        s = np.random.randint(0, 2)
        if s == 0:
            data_day = time_handle.time_handling(
                t, i, n_days, t1, my, sigma, interpolated, t1_we, my1_we, sigma_we, interpolated_we)
            data_final += data_day
        else:
            data_day = time_handle.time_handling(
                t, i, n_days, t1, my, sigma, interpolated, t1_we, my2_we, sigma_we, interpolated_we)
            data_final += data_day

    data_final = np.transpose(data_final)

    return data_final
